# SCRIPT FOR SENDING PUSHOVER NOTIFICATIONS WITH A CUSTOM RESIZED ALERT IMAGE ATTACHMENT
#
# Version: 1.9
# Created by jaydeel at ipcamtalk.com
# https://ipcamtalk.com/threads/send-pushover-notifications-with-pictures-and-hyperlinks.58819/post-769688
#
# Updates:
# V1.9  - 2023-07-12 - fixed bug with Pushover -Priority parameter (mis-typed as [string] instead of [int])
#                    - reworked function 'PurgeOldFile()'
# V1.8  - 2023-06-04 - added options to tune image resizing; 'WxH' (default), 'Quality' and 'Hybrid'; see USER SETTINGS $targetkb_method
#                    - added new named argument '-Demo Y' to suspend notifications & BI logfile posting (useful for tuning resize settings)
#                    - added automatic renaming of user settings file; changed hard-coded filename, now using 'THIS_SCRIPTNAME_user_settings.ps1'
#                    - fixed bugs for arguments -Quality, -Width & -TargetKB (conditionals incorrectly assumed [int] typing)
#                    - fixed bugs that prevented the posting of (some) error messages to the Blue Iris logfile
# V1.7  - 2023-05-31 - fixed bugs & clarified user settings comments for posting messages to the Blue Iris logfile
# V1.6  - 2023-05-30 - moved USER SETTINGS code to a separate configuration file (pushover_resize_user_settings.ps1)
#                    - added named argument '-Title' (for custom notification titles)
#                    - added 5 user settings to support posting error messages to the Blue Iris logfile
#                      (use setting '$post_to_BI_logfile_flg' to enable/disable this feature)
#                    - added a user setting to post ALL resize event messages to the Blue Iris logfile ($post_all_events)
#                    - improved background messaging toggling
# V1.5  - 2023-05-28 - added 7 named commandline arguments: -Msg, -Priority, -AttachFile, -Quality, -Width, -TargetKB -Quiet
#                      using any of the last five will override the defaults in the USER SETTINGS section
#                    - changed user setting '$img_size' to '$img_width' (for clarity of its function)
#                    - fixed potential memory leak opportunity in function ResizeImage()
#                    - added a background message that summarizes attachment resize results and script performance
#                    - added WxHBefore and WxHAfter columns to CSV log (must rename existing CSV file before they will appear)
# V1.4  - 2023-05-27 - changed attachment argument requirement to optional; script now requires only message argument
# V1.3  - 2023-05-26 - added Powershell's execution policy info to the background message
#                    - added alert jpg's filename to the background message
#                    - added '-UseQuotes AsNeeded' to 'Export-Csv' cmdlet in function LogPerformance()
# V1.2  - 2023-05-25 - added user setting for background messaging
# V1.1  - 2023-05-24 - fixed bug in function ResizeImage()
# V1    - 2023-05-24 - initial release
#
# USAGE:      {this script name}.ps1 Msg [Priority AttachFile Quality Width, TargetKB Quiet]
# ARGUMENTS:  Msg        = [REQUIRED] [string] Pushover notification message - may contain HTML
#             Priority   = [OPTIONAL] (integer, or '*') -2|-1|0|1|2; -2=lowest, -1=low(silent), 0=normal (default), 1=high-priority, 2=emergency
#             AttachFile = [OPTIONAL] (string) image filename (typ. in ALERTS folder), typ. via Blue Iris macro &ATTACH_PATH
#             Quality    = [OPTIONAL] (integer, or '*') target image quality 1-100; units %
#             Width      = [OPTIONAL] (integer, or '*') target image size, smallest axis; units pixels
#             TargetKB   = [OPTIONAL] (integer, or '*') target filesize in kilobytes
#             Title      = [OPTIONAL] (string) custom title for pushover message
#             Quiet      = [OPTIONAL] (string) 'Y' or 'N') 'Y' disables background messaging
#             Demo       = [OPTIONAL] (string) 'Y' or 'N') 'Y' disables sending notifications, and posing to the Blue logfile
#
# EXAMPLES:
# 	Send notification without an attachment
#   	pushover_resize.ps1 'test msg'
#   	pushover_resize.ps1 -Msg 'test msg' -Title "test title"
# 	Send notification without an attachment with custom title and priority
#   	pushover_resize.ps1 -Msg 'test msg' -Title "test title" -Priority -1
# 	Send notification with default priority and with attachment as-s (not resized)
#   	pushover_resize.ps1 'test msg' * 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' 100 0
#   	pushover_resize.ps1 -Msg 'test msg' -Title "test title" -AttachFile 'D:\Blue Iris\Alerts\DW1*.jpg' -Quality 100 -Width 0
# 	Change only img_quality, use default img_width
#   	pushover_resize.ps1 'test msg' * 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' 70
# 	Change only img_width, use default img_quality
#		pushover_resize.ps1 'test msg' * 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' * 500
#		pushover_resize.ps1 -Msg 'test msg' 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' -Width 500
# 	Change both img_quality and img_width
#		pushover_resize.ps1 'test msg' * 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' 70 500
# 	Target attachment size of 500 KB, using default img_quality & img_width
#		pushover_resize.ps1 'test msg' * 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' * * 500
#		pushover_resize.ps1 -Msg 'test msg' -Priority * -AttachFile 'D:\Blue Iris\Alerts\DW1*.jpg' -TargetKB 500
# 	Target attachment size of 500 KB, using specified img_quality
#		pushover_resize.ps1 'test msg' * 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' 70 * 500
#		pushover_resize.ps1 -Msg 'test msg' -AttachFile 'D:\Blue Iris\Alerts\DW1.20230521_160000.3076088.17-0.jpg' -Quality -TargetKB 500

# MUST BE THE FIRST LINE IN A SCRIPT
param(
	[string]$Msg,         # [REQUIRED]
	[int]$Priority,       # [OPTIONAL]
	[string]$AttachFile,  # [OPTIONAL]
	[string]$Quality,     # [OPTIONAL]
	[string]$Width,       # [OPTIONAL]
	[string]$TargetKB,    # [OPTIONAL]
	[string]$Title,       # [OPTIONAL]
	[string]$Quiet,       # [OPTIONAL]
	[string]$Demo         # [OPTIONAL]
)

if([string]::IsNullOrWhiteSpace($Priority)) {
   $Priority = 0
}
if([string]::IsNullOrWhiteSpace($Quality)) {
   $Quality = "*"
}
if([string]::IsNullOrWhiteSpace($Width)) {
   $Width = "*"
}
if([string]::IsNullOrWhiteSpace($TargetKB)) {
   $TargetKB = "*"
}
if([string]::IsNullOrWhiteSpace($Title)) {
   $Title = "*"
}
if([string]::IsNullOrWhiteSpace($Quiet)) {
   $Quiet = "N"
}
if([string]::IsNullOrWhiteSpace($Demo)) {
   $Demo = "N"
}

# For debugging
#Write-Host "Title = $Title"
#Write-Host "Msg = $Msg"
#Write-Host "Priority = $Priority"
#Write-Host "AttachFile = $AttachFile"
#Write-Host "Quality = $Quality"
#Write-Host "Width = $Width"
#Write-Host "TargetKB = $TargetKB"
#Write-Host "Quiet = $Quiet"
#Write-Host "Demo = $Demo"

#Set-StrictMode  -Version 3
#Start-Transcript  # default destination is C:\Users\{user}l\Documents

# GATHER SCRIPT INFO 
$ScriptName = $MyInvocation.MyCommand.Name
$ScriptPath = Split-Path $MyInvocation.MyCommand.Path

# ================================================
# USER SETTINGS...

# Import external 'user settings' script (expected to be in same folder as this script)
$user_settings_filename = $ScriptName.Replace(".ps1", "_user_settings.ps1")
try {
	. $user_settings_filename
	
	# For debugging
	#Write-Host "Loaded user settings from '$user_settings_filename'"
}
catch {
	return "ERR/failed to load user settings file '$user_settings_filename'`nTip: try executing it directly to confirm it exists and reveal syntax errors"
}

# END USER SETTINGS
# ===============================================

$variable_test = Get-Variable -Name bg_msgs_flg -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$bg_msgs_flg = $true
}
$variable_test = Get-Variable -Name targetkb_method -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$targetkb_method = 'WxH'
}
if (@('WxH','Quality','Hybrid') -notcontains $targetkb_method) {
	$targetkb_method = 'WxH'
}
$variable_test = Get-Variable -Name targetkb_wxh_step -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$targetkb_wxh_step = 25
}
$variable_test = Get-Variable -Name targetkb_quality_step -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$targetkb_quality_step = 20
}
$variable_test = Get-Variable -Name img_quality -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$img_quality = 90
}
$variable_test = Get-Variable -Name img_width -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$img_width = 500
}
$variable_test = Get-Variable -Name img_kilobytes -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$img_kilobytes = 0
}
$variable_test = Get-Variable -Name img_path -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$img_path = ""
} else {
	if (!$img_path.EndsWith('\')) {
		$img_path = $img_path + '\'
	}
}
$variable_test = Get-Variable -Name img_name_suffix -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$img_name_suffix = 'resized'
}
if ($img_path -eq "") {
	# Necessary to prevent overwriting the original alert files (typ. in the Blue Iris 'Alerts' folder)
	$img_name_suffix = 'resized'
}
$variable_test = Get-Variable -Name img_lifetime_hours -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$img_lifetime_hours = 24
}
$variable_test = Get-Variable -Name post_to_BI_logfile_flg -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$post_to_BI_logfile_flg = $false
}
$variable_test = Get-Variable -Name post_all_events -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -eq $null) {
	$post_all_events = $false
}

# INITIALIZE VARS
$PsVersion = $PSVersionTable.PSVersion.Major
# NOTE: Scripts running from Windows Task Scheduler have a parent process named 'svchost'.
#       https://stackoverflow.com/a/65894924
$ParentProcess = (Get-Process -Id (Get-CimInstance Win32_Process -Filter "ProcessID = $pid").ParentProcessId).Name
# NOTE: Scripts called by another script have a 'PSCommandPath'.
#       Method 'PSCommandPath' returns null if script is executed via the terminal
#       https://stackoverflow.com/a/23370618/13499984
$CallingScript = $MyInvocation.PSCommandPath

# DETERMINE IF RUNNING IN BACKGROUND (IN CLI MODE, OR FROM BLUE IRIS)
if ($ParentProcess -eq 'BlueIris') {
	$fromConsole = $false
} else {
	$fromConsole = ($CallingScript -eq $null)
}

# For debugging
#msg.exe * /TIME:10 "entered script '$ScriptName'"
#msg.exe * /TIME:10 "PsVersion ($ScriptName) = $PsVersion"
#msg.exe * /TIME:10 "ParentProcess ($ScriptName) = $ParentProcess"
#msg.exe * /TIME:10 "fromConsole ($ScriptName) = $fromConsole"

# SHOW OPTIONAL MESSAGE WHEN RUNNING FROM BACKGROUND
if (!$fromConsole) {
	if ($bg_msgs_flg ) {
		$ExecPolicy = Get-ExecutionPolicy
		$exec_policy_str = "PSExecutionPolicy = $ExecPolicy"
		if ($env:PSExecutionPolicyPreference) {
			$ExecPolicyPref = $env:PSExecutionPolicyPreference
			$exec_policy_str = "Session ExecutionPolicy = $ExecPolicyPref"
		}
		$msg_sec = 10
		$bg_msg = "Script '$ScriptName' is running in the background" + 
		"`n$exec_policy_str" + 
		"`nPsVersion = $PsVersion" + 
		"`nParentProcess = $ParentProcess" + 
		"`nalert = $AttachFile" + 
		"`n`n(this msg auto-closes in $msg_sec seconds)"
		msg.exe * /TIME:$msg_sec $bg_msg
	}
}

# For testing CLI mode
#$fromConsole = $false

# ===============================================

function PostCustomBiLogEntry() {
# PURPOSE: Add custom message to the Blue Iris logfile
# ARGS:    $MsgStr        = (string) may contain HTML
#          $Level         = (integer) 0|1|2 (0:info, 1:warn, 2:error)
#          $SilentFlg     = (boolean) $true prevents background messaging
# RETURNS: nothing

    param(
		[string]$MsgStr,      # may contain HTML
		[string]$Level,       # full filespec of resized alert image
		[boolean]$SilentFlg   # 
	)

	# For debugging
	#Write-Host "SilentFlg = $SilentFlg"
	
	# Remove/replace incompatible substrings from $MsgStr
	$MsgStr = $MsgStr -Replace "`n", "%20" -Replace "\\r",  "%20" -Replace "\\n", "%20" -Replace " ", "%20"
	
	$bi_address = $script:bi_webserver
	if ($script:post_w_creds_flg) {
		$uri = "http://$bi_address/admin?log=$MsgStr&level=$Level&user=$script:bi_user_name&pw=$script:bi_user_pw"
	} else {
		$uri = "http://$bi_address/admin?log=$MsgStr&level=$Level"
	}
	$result = Invoke-WebRequest -Uri $uri
	
	# For debugging
	#Write-Host "uri = $uri"
	#Write-Host "result:`n$result"	

	# HANDLE NOTIFICATIONS
	if (!$SilentFlg) {
		msg.exe * /TIME:10 "Custom BI logfile message posted...`nlevel = $Level`n'$MsgStr'"
	}
	if ($script:fromConsole) {
		Write-Host ("`nCustom BI logfile message posted:`n'" + ($MsgStr.Replace("%20"," ")) + "'`n")
	}
}

function ResizeImage() {
# PURPOSE:  Resize image file to be used as attachment for Pushover notification 
# ARGS:     see param lines below
# RETURNS:  (array) @(before_KB, after_KB, before_WH, after_WH)
# NOTES:    Be sure to check if 'after_KB' = 0, as this indicates the new file was not actually created
# ATTRIBUTIONS:
#   Derived from: https://www.lewisroberts.com/2015/01/18/powershell-image-resize-function/
#   Also used: https://gist.github.com/someshinyobject/617bf00556bc43af87cd
	
    param(
		[string]$SourceFilespec,          # full filespec
		[string]$NewImagePath,            # folder location for resized files
		[int]$Quality = 90,               # integer 0 to 100
		[int]$Width = 500,                # pixels
		[int]$TargetKb,                   # kilobytes; NOTE: Pushover API max is 2,621,440 bytes (2621 KB)
		[string]$TargetKbMethod = 'WxH',  # 'WxH', 'Quality', or 'Hybrid'
		[int]$TargetKbWxHDelta = 25,      # percentage to decrease WxH per iteration
		[int]$TargetKbQualityDelta = 25,  # percentage to decrease Quality per iteration
		[boolean]$SilentFlg               # $true prevents background messaging (showing file resizing results)
	)
	
	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#Write-Host "Entered function = $fn_name()"	
	#Write-Host "$fn_name() SourceImagePath = '$SourceFilespec'"
	#Write-Host "$fn_name() NewImagePath = '$NewImagePath'"
	#Write-Host "$fn_name() Quality = $Quality"
	#Write-Host "$fn_name() Width = $Width"
	#Write-Host "$fn_name() TargetKb = $TargetKb"
	#Write-Host "$fn_name() TargetKbMethod = $TargetKbMethod"
	#Write-Host "$fn_name() TargetKbWxHDelta = $TargetKbWxHDelta"
	#Write-Host "$fn_name() TargetKbQualityDelta = $TargetKbQualityDelta"
	
	if (-not(Test-Path -LiteralPath "$SourceFilespec") ) {
		$oldSizeKb = $null
	}

	$OutputLocation = $NewImagePath
	
    Add-Type -AssemblyName "System.Drawing"
 
    $img = [System.Drawing.Image]::FromFile($SourceFilespec)
 
	$oldHeight = $img.Height
	$oldWidth  = $img.Width
	$oldSizeKb = [math]::Round((Get-Item -Path $SourceFilespec).Length/1kB,1).ToString("#.0")
	$oldWH = "$oldWidth x $oldHeight"
		
	if ($script:fromConsole) {
		Write-Host "SourceImagePath = '$SourceFilespec'"
		Write-Host "orig height/width/size(KB) = $oldHeight/$oldWidth/$oldSizeKb"
	}
	
	if ($TargetKb -gt 0) {
		# SPECIAL CASE - ITERATE RESIZING UNTIL FILE SIZE IS BELOW TARGET KILOBYTES
					
		if ($script:fromConsole) {
			Write-Host "iterating to achieve file size < $TargetKb KB (using method '$TargetKbMethod')..."
		}
		
		if ($TargetKbMethod -eq 'WxH') {
			$i = 1
			$step = $TargetKbWxHDelta
			for ($x=(100-$step); $x -gt 0; $x=$x-$step) {
				
				# Show progress
				if ($script:fromConsole) {
					Write-Host ("trying size reduction = -" + (100-$x) + "%")
				}
				
				$k = $x/100
				$CanvasHeight = $oldHeight*$k
				$CanvasWidth  = $oldWidth*$k

				# Encoder parameter for image quality
				$ImageEncoder = [System.Drawing.Imaging.Encoder]::Quality
				$encoderParams = New-Object System.Drawing.Imaging.EncoderParameters(1)
				$encoderParams.Param[0] = New-Object System.Drawing.Imaging.EncoderParameter($ImageEncoder, $Quality)
			 
				# Get codec
				$Codec = [System.Drawing.Imaging.ImageCodecInfo]::GetImageEncoders() | Where {$_.MimeType -eq 'image/jpeg'}
			 
				# Compute the final ratio to use
				$ratioX = $CanvasWidth / $img.Width;
				$ratioY = $CanvasHeight / $img.Height;
			 
				$ratio = $ratioY  # Default
				if ($ratioX -le $ratioY) {
					$ratio = $ratioX
				}
			 
				$newWidth = [int] ($img.Width * $ratio)
				$newHeight = [int] ($img.Height * $ratio)

				$bmpResized = New-Object System.Drawing.Bitmap($newWidth, $newHeight)
				#$bmpResized = New-Object System.Drawing.Bitmap -ArgumentList $newWidth, $newHeight
				$graph = [System.Drawing.Graphics]::FromImage($bmpResized)
				$graph.InterpolationMode = [System.Drawing.Drawing2D.InterpolationMode]::HighQualityBicubic
			 
				$graph.Clear([System.Drawing.Color]::White)
				$graph.DrawImage($img, 0, 0, $newWidth, $newHeight)
			 
				# Save to file
				$bmpResized.Save($OutputLocation, $Codec, $($encoderParams))
				$bmpResized.Dispose()
			
				if (-not(Test-Path -LiteralPath "$NewImagePath") ) {
					$err_msg = "ERR/Resize failed; cannot find NewImagePath '$NewImagePath'"
					if ($scriot:post_to_BI_logfile_flg) {
						PostCustomBiLogEntry -MsgStr "$err_msg ($script:ScriptName)" -Level 2 -SilentFlg $SilentFlg
					}
					if ($script:fromConsole) {
						Write-Host $err_msg
					}
					msg.exe * /TIME:10 $err_msg
					
					# Release resource to avoid memory leak
					$img.Dispose()  
					
					return @($oldSizeKb, 0, $oldWH, "")

				} else {
					$newSizeKb = [math]::Round((Get-Item -Path $NewImagePath).Length/1kB, 1)
					$newWH = "$newWidth x $newHeight"
				}
				
				# Show progress
				if ($script:fromConsole) {
					Write-Host "--- result #$i height/width/size(KB) = $newHeight/$newWidth/$newSizeKb"
				}

				if ($newSizeKb -lt [int]$TargetKb) {
					if ($script:fromConsole) {
						Write-Host "achieved target after $i iterations"
					}
					break				
				} else {
					if ($x -le $step) {
						if ($script:fromConsole) {
							Write-Host ("unable to achieve target after $i iterations; using final try results")
						}
					}
				}	
				$i++
			}
		}
		
		if ($TargetKbMethod -eq 'Quality') {
			$i = 1
			$step = $TargetKbQualityDelta  # percentage to decrease WxH per iteration
			for ($q=($Quality-$step); $q -gt 0; $q=$q-$step) {
				
				# Show progress
				if ($script:fromConsole) {
					Write-Host ("trying quality reduction = -" + ($i*$step) + "%")
				}

				# Encoder parameter for image quality
				$ImageEncoder = [System.Drawing.Imaging.Encoder]::Quality
				$encoderParams = New-Object System.Drawing.Imaging.EncoderParameters(1)
				$encoderParams.Param[0] = New-Object System.Drawing.Imaging.EncoderParameter($ImageEncoder, $q)
			 
				# Get codec
				$Codec = [System.Drawing.Imaging.ImageCodecInfo]::GetImageEncoders() | Where {$_.MimeType -eq 'image/jpeg'}
				$newWidth = $img.Width
				$newHeight = $img.Height

				$bmpResized = New-Object System.Drawing.Bitmap($newWidth, $newHeight)
				#$bmpResized = New-Object System.Drawing.Bitmap -ArgumentList $newWidth, $newHeight
				$graph = [System.Drawing.Graphics]::FromImage($bmpResized)
				$graph.InterpolationMode = [System.Drawing.Drawing2D.InterpolationMode]::HighQualityBicubic
			 
				$graph.Clear([System.Drawing.Color]::White)
				$graph.DrawImage($img, 0, 0, $newWidth, $newHeight)
			 
				# Save to file
				$bmpResized.Save($OutputLocation, $Codec, $($encoderParams))
				$bmpResized.Dispose()
			
				if (-not(Test-Path -LiteralPath "$NewImagePath") ) {
					$err_msg = "ERR/Resize failed; cannot find NewImagePath '$NewImagePath'"
					if ($scriot:post_to_BI_logfile_flg) {
						PostCustomBiLogEntry -MsgStr "$err_msg ($script:ScriptName)" -Level 2 -SilentFlg $SilentFlg
					}
					if ($script:fromConsole) {
						Write-Host $err_msg
					}
					msg.exe * /TIME:10 $err_msg
					
					# Release resource to avoid memory leak
					$img.Dispose()  
					
					return @($oldSizeKb, 0, $oldWH, "")
					
				} else {
					$newSizeKb = [math]::Round((Get-Item -Path $NewImagePath).Length/1kB, 1)
					$newWH = "$newWidth x $newHeight"
				}
				
				# Show progress
				if ($script:fromConsole) {
					Write-Host "--- result #$i height/width/size(KB) = $newHeight/$newWidth/$newSizeKb"
				}

				if ($newSizeKb -lt [int]$TargetKb) {
					if ($script:fromConsole) {
						Write-Host "achieved target after $i iterations"
					}
					break				
				} else {
					if ($q -le $step) {
						if ($script:fromConsole) {
							Write-Host ("unable to achieve target after $i iterations; using final try results")
						}
					}
				}	
				$i++
			}
		}
			
		if ($TargetKbMethod -eq 'Hybrid') {
			$i = 1
			$step = $TargetKbWxHDelta
			for ($x=(100-$step); $x -gt 0; $x=$x-$step) {
				
				# Show progress
				if ($script:fromConsole) {
					Write-Host ("trying hybrid reduction: size = -" + (100-$x) + "%; quality = -" + ($i*$TargetKbQualityDelta) + "%")
				}
				
				$j = $x/100
				$CanvasHeight = $oldHeight*$j
				$CanvasWidth  = $oldWidth*$j
				$q = $Quality - $i*$TargetKbQualityDelta

				# Encoder parameter for image quality
				$ImageEncoder = [System.Drawing.Imaging.Encoder]::Quality
				$encoderParams = New-Object System.Drawing.Imaging.EncoderParameters(1)
				$encoderParams.Param[0] = New-Object System.Drawing.Imaging.EncoderParameter($ImageEncoder, $q)
			 
				# Get codec
				$Codec = [System.Drawing.Imaging.ImageCodecInfo]::GetImageEncoders() | Where {$_.MimeType -eq 'image/jpeg'}
			 
				# Compute the final ratio to use
				$ratioX = $CanvasWidth / $img.Width;
				$ratioY = $CanvasHeight / $img.Height;
			 
				$ratio = $ratioY  # Default
				if ($ratioX -le $ratioY) {
					$ratio = $ratioX
				}
			 
				$newWidth = [int] ($img.Width * $ratio)
				$newHeight = [int] ($img.Height * $ratio)

				$bmpResized = New-Object System.Drawing.Bitmap($newWidth, $newHeight)
				#$bmpResized = New-Object System.Drawing.Bitmap -ArgumentList $newWidth, $newHeight
				$graph = [System.Drawing.Graphics]::FromImage($bmpResized)
				$graph.InterpolationMode = [System.Drawing.Drawing2D.InterpolationMode]::HighQualityBicubic
			 
				$graph.Clear([System.Drawing.Color]::White)
				$graph.DrawImage($img, 0, 0, $newWidth, $newHeight)
			 
				# Save to file
				$bmpResized.Save($OutputLocation, $Codec, $($encoderParams))
				$bmpResized.Dispose()
			
				if (-not(Test-Path -LiteralPath "$NewImagePath") ) {
					$err_msg = "ERR/Resize failed; cannot find NewImagePath '$NewImagePath'"
					if ($scriot:post_to_BI_logfile_flg) {
						PostCustomBiLogEntry -MsgStr "$err_msg ($script:ScriptName)" -Level 2 -SilentFlg $SilentFlg
					}
					if ($script:fromConsole) {
						Write-Host $err_msg
					}
					msg.exe * /TIME:10 $err_msg
					
					# Release resource to avoid memory leak
					$img.Dispose()  
					
					return @($oldSizeKb, 0, $oldWH, "")
					
				} else {
					$newSizeKb = [math]::Round((Get-Item -Path $NewImagePath).Length/1kB, 1)
					$newWH = "$newWidth x $newHeight"
				}
				
				# Show progress
				if ($script:fromConsole) {
					Write-Host "--- result #$i height/width/size(KB) = $newHeight/$newWidth/$newSizeKb"
				}

				if ($newSizeKb -lt [int]$TargetKb) {
					if ($script:fromConsole) {
						Write-Host "achieved target after $i iterations"
					}
					break				
				} else {
					if (($x -le $step) -or ($q -le $TargetKbQualityDelta)) {
						if ($script:fromConsole) {
							Write-Host ("unable to achieve target after $i iterations; using final try results")
						}
					}
					# Look ahead (necessary b/c $q is not managed by for:next loop)
					if (($q - $TargetKbQualityDelta) -lt $TargetKbQualityDelta) {
						break
					}
				}	
				$i++
			}
		}

		if ($script:fromConsole) {
			Write-Host "new height/width/size(KB) = $newHeight/$newWidth/$newSizeKb"
			Write-Host "NewImagePath = '$NewImagePath'"
		}
				
		# Release resource to avoid memory leak
		$img.Dispose()  
		
		return @($oldSizeKb, $newSizeKb.ToString("#.0"), $oldWH, $newWH)

	} else {	
		if (($Quality -eq 100) -and ($Width -eq 0)) {

			# SPECIAL CASE - USE THE SOURCE IMAGE AS-IS

			$newHeight = $oldHeight
			$newWidth  = $oldWidth
			$newSizeKb = $oldSizeKb
			
			# Copy the original image file 'as-is'
			Copy-Item -LiteralPath "$SourceFilespec" -Destination "$NewImagePath"			
			# ... and give it a current timestamp
			#     WHY? w/o a new timestamp, it may be deleted by 'PurgeOldFiles()'
			$file = Get-Item "$NewImagePath"
			$file.LastWriteTime = (Get-Date)	

		} else {

			# NORMAL CASE - RESIZE THE SOURCE IMAGE

			$CanvasWidth  = $Width
			$CanvasHeight = ($Width / $oldWidth) * $oldHeight
		 
			# Encoder parameter for image quality
			$ImageEncoder = [System.Drawing.Imaging.Encoder]::Quality
			$encoderParams = New-Object System.Drawing.Imaging.EncoderParameters(1)
			$encoderParams.Param[0] = New-Object System.Drawing.Imaging.EncoderParameter($ImageEncoder, $Quality)
		 
			# Get codec
			$Codec = [System.Drawing.Imaging.ImageCodecInfo]::GetImageEncoders() | Where {$_.MimeType -eq 'image/jpeg'}
		 
			# Compute the final ratio to use
			$ratioX = $CanvasWidth / $img.Width;
			$newWidth  = [int] ($img.Width * $ratioX)
			$newHeight = [int] ($img.Height * $ratioX)
			
			#Write-Host "newWidth = $newWidth"
			#Write-Host "newHeight = $newHeight"

			$bmpResized = New-Object System.Drawing.Bitmap($newWidth, $newHeight)
			#$bmpResized = New-Object System.Drawing.Bitmap -ArgumentList $newWidth, $newHeight
			$graph = [System.Drawing.Graphics]::FromImage($bmpResized)
			$graph.InterpolationMode = [System.Drawing.Drawing2D.InterpolationMode]::HighQualityBicubic
		 
			$graph.Clear([System.Drawing.Color]::White)
			$graph.DrawImage($img, 0, 0, $newWidth, $newHeight)
		 
			# Save to file
			$bmpResized.Save($OutputLocation, $Codec, $($encoderParams))
			$bmpResized.Dispose()
			$img.Dispose()
		
			if (-not(Test-Path -LiteralPath "$NewImagePath") ) {
				$err_msg = "ERR/Resize failed; cannot find NewImagePath '$NewImagePath'"
				if ($script:post_to_BI_logfile_flg) {
					PostCustomBiLogEntry -MsgStr "$err_msg ($script:ScriptName)" -Level 2 -SilentFlg $SilentFlg
				}
				if ($script:fromConsole) {
					Write-Host $err_msg
				}
				msg.exe * /TIME:10 $err_msg
				
				# Release resource to avoid memory leak
				#$img.Dispose()  

				return @($oldSizeKb, $null)
			} else {
				$newSizeKb = [math]::Round((Get-Item -Path $NewImagePath).Length/1kB, 1).ToString("#.0")
			}
		}
		if ($script:fromConsole) {
			Write-Host "NewImagePath = '$NewImagePath'"
			Write-Host "new height/width/size(KB) = $newHeight/$newWidth/$newSizeKb"
		}
	}
	
	$oldWH = "$oldWidth x $oldHeight"
	$newWH = "$newWidth x $newHeight"
	
	# For debugging
	#Write-Host "oldWH = $oldWH"
	#Write-Host "newWH = $newWH"
	
	# Release resource to avoid memory leak
	#$img.Dispose()
	
	return @($oldSizeKb, $newSizeKb, $oldWH, $newWH)
}

function SendPushover() {
# PURPOSE: Send custom notification via the Pushover API
# ARGS:    $MsgTitle      = (string) pushover notification title
#          $MsgStr        = (string) pushover notification message; may contain HTML
#          $AttachPath    = (string) full filespec of resized alert image, or $null
#          $PriorityLevel = (integer) -2|-1|0|1|2; -2=lowest, -1=low(silent), 0=normal, 1=high-priority, 2=emergency (default=0)
#          $SilentFlg     = (boolean) $true prevents background messaging (showing the message sent)
# RETURNS: nothing
# ATTRIBUTIONS:
#    handling attachments: https://tinyurl.com/mrxp7pn3  (PoshOver Powershell Package)
	
    param(
		[string]$MsgTitle,    # may contain HTML
		[string]$MsgStr,      # may contain HTML
		[string]$AttachPath,  # full filespec of resized alert image
		[int]$PriorityLevel,  # -2|-1|0|1|2
		[boolean]$SilentFlg
	)
	
	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#Write-Host "Entered function = $fn_name()"
	#Write-Host "$fn_name() MsgTitle = '$MsgTitle'"
	#Write-Host "$fn_name() MsgStr = '$MsgStr'"
	#Write-Host "$fn_name() AttachPath = '$AttachPath'"	
	#Write-Host "$fn_name() PriorityLevel = PriorityLevel"
	#Write-Host "$fn_name() SilentFlg = $SilentFlg"
	#Write-Host "$fn_name() fromConsole = $script:fromConsole"
	# NOTE REGARDGING MSG.EXE COMMANDS
	# ... You may need to replace 'msg.exe *' with 'msg.exe /server:computername username
	#msg.exe * /TIME:20 "Pushover notification args...`npriority_level = $PriorityLevel`nSilentFlg = $SilentFlg`nAttachPath = $AttachPath`nMsgStr = $MsgStr"
	
	#Prep the attachment
	if ($AttachPath) {
		#Get-Item -Path $AttachPath
		$attach_obj = (Get-Item -Path $AttachPath)
		$attach_bytes = [System.IO.File]::ReadAllBytes($AttachPath)
	}

	# For debugging
	#Write-Host ("attach_obj = " + $attach_obj)
	#Write-Host ("attach_obj.Name = " + $attach_obj.Name)
	
	$uri = 'https://api.pushover.net/1/messages.json'
	$parameters = @{
		token      = $script:po_app_token
		user       = $script:po_user_token
		priority   = $PriorityLevel
		retry      = $script:po_retry
		expire     = $script:po_expire
		tags       = $script:po_tags
		sound      = $script:po_sound
		device     = $script:po_device
		html       = 1
		title      = $MsgTitle
		message    = $MsgStr
	}
	
	# For debugging
	#Write-Host ("parameters.Keys = " + $parameters.Keys)
	#Write-Host "`nPushover parameters..."
	[string[]]$arr = $parameters | Out-String -stream
	foreach ($item in $arr) {
		if ($item) {
			#Write-Host $item
		}
	}
	
	# EXECUTE
	if (!$AttachPath -or ($attach_bytes.Length -eq 0)) {
		# SEND BASIC NOTIFICATION

		# ... NOTE: When running in the background, both Invoke-RestMethod & Invoke-WebRequest
		#	  do not execute; it doesn't produce anything, no error, warning, or output.
		#     REF: https://stackoverflow.com/a/50274056 looked like a possible solution
		#     however, since PS v6.0 switch -UseBasicParsing has been deprecated, and 
		#     all web requests now use basic parsing only.
		# ... attachmnets
		#     https://psg-prod-centralus.cloudapp.net/packages/Poshover/0.1.450600-alpha/Content/Private%5CSend-MessageWithAttachment.ps1
		$Response = $parameters | Invoke-RestMethod -Uri $uri -Method Post
		# Alternative syntaxes
		#$parameters | Invoke-RestMethod -Uri $uri -Method Post | Out-Null
		#$Response = Invoke-RestMethod -Uri $uri -Method Post -Form $parameters

	} else {	
		# SEND NOTIFICATION WITH ATTACHMENT
		
		# Notes:
		# Borrowed code from:  https://github.com/joshooaj/Poshover/blob/main/src/Private/Send-MessageWithAttachment.ps1
		# More refs:           https://gist.github.com/markekraus/a0135d0a5f52b609eda513df4db71cab

		try {

            Add-Type -AssemblyName System.Net.Http
			# NOTE: Originally, the $client= line below was not working when this script was 
			#        executed in the background via a Blue Iris 'run a program' action
			#        SOLUTION:  https://stackoverflow.com/a/70535083
			$client = [system.net.http.httpclient]::new()

			try {
				$content = [system.net.http.multipartformdatacontent]::new()
				foreach ($key in $parameters.Keys) {
					$textContent = [system.net.http.stringcontent]::new($parameters.$key)
					$content.Add($textContent, $key)
				}
				
				# For debugging
				#Write-Host ("content = " + $content | ConvertTo-Json)
				
				$jpegContent = [system.net.http.bytearraycontent]::new($attach_bytes)
				$jpegContent.Headers.ContentType = [system.net.http.headers.mediatypeheadervalue]::new('image/jpeg')
				$jpegContent.Headers.ContentDisposition = [system.net.http.headers.contentdispositionheadervalue]::new('form-data')
				$jpegContent.Headers.ContentDisposition.Name = 'attachment'
				$jpegContent.Headers.ContentDisposition.FileName = $AttachPath
				$content.Add($jpegContent)

				# Not needed.
				# REF: https://stackoverflow.com/questions/43014894/httpclient-and-stream-issues
				#$content.LoadIntoBufferAsync().GetAwaiter().GetResult()

				# Modify the message body
				Write-Verbose "Message body:`r`n$($content.ReadAsStringAsync().Result.Substring(0, 2000).Replace($parameters.token, "********").Replace($parameters.user, "********"))"

				# DO IT!
				#msg.exe * /TIME:10 "executing pushover api cmd w/ attachment..."
				$Response = $client.PostAsync($uri, $content).Result
				
				if ($script:fromConsole) {
					Write-Output ($Response.Content.ReadAsStringAsync().Result | ConvertFrom-Json)
				}
			}
			finally {
				$content.Dispose()
			}
		}
		finally {
			$client.Dispose()
		}
	}

	# PROVIDE FEEDBACK
	if (!$SilentFlg) {
		#msg.exe * /TIME:15 "Pushover notification sent...`npriority_level = $PriorityLevel`n$MsgStr"
		#msg.exe * /TIME:10 "Pushover notification attachment...`n$AttachPath"
		#msg.exe * /TIME:30 ("Pushover notification response...`n" + ($Response | ConvertTo-Json))
		#msg.exe * /TIME:30 ("Pushover notification response...`n" + $Response)
	}
	
	# For debugging
	#if (!((($Response.Content.ReadAsStringAsync().Result | ConvertFrom-Json)) -match "invalid")) {
		
	if ((($Response.Content.ReadAsStringAsync().Result | ConvertFrom-Json)) -match "invalid") {
		$err_msg = "ERR/Pushover API returned 'invalid'"
		if ($script:post_to_BI_logfile_flg) {
			PostCustomBiLogEntry -MsgStr "$err_msg ($script:ScriptName)" -Level 2 -SilentFlg $SilentFlg
		}
		if ($script:fromConsole) {
			Write-Host "$err_msg`n"
		}
		if (!($SilentFlg)) {
			msg.exe * /TIME:10 "$err_msg ($script:ScriptName)"			
		}
	} else {
		if ($script:fromConsole) {
			Write-Host "Pushover notification sent`n"
		}
	}
}

function LogPerformance() {
# PURPOSE:  Log performance data to CSV file for image file resizing and Pushover notification actions.
# ARGS:     see param lines below
# RETURNS:  nothing
# NOTES:    1. CSV file is created automatically if it does not already exist. 
#           2. CSV file's header row is generated from the object keys in variable $new_row_obj.
	
    param(
		[string]$csv_filespec, 
		[string]$img_filename, 
		[string]$timestamp, 
		[double]$before_kb, 
		[double]$after_kb, 
		[string]$before_wh, 
		[string]$after_wh, 
		[double]$total_ms, 
		[double]$resize_ms, 
		[double]$pusho_ms
	)
	
	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#Write-Host "Entered function = $fn_name()"
		
	# Build the new row (object keys are the header column labels)
	$new_row_obj = [PSCustomObject]@{
		'Timestamp'    = $timestamp
		'SizeBeforeKb' = $before_kb
		'SizeAfterKb'  = $after_kb
		'WxHBefore'    = $before_wh
		'WxHAfter'     = $after_wh
		'TotalMsec'    = $total_ms
		'ResizeMsec'   = $resize_ms
		'PushoMsec'    = $pusho_ms
		'ImageFile'    = $img_filename
	}
	
	# For debugging
	#msg.exe * /TIME:10 "new_row_obj... `n$new_row_obj"

	# Append new row to CSV file, creating the file if necessary
	if ($PSVersionTable.PSVersion.Major -ge 7) {
		# REFS: https://tinyurl.com/yvy4mjy6
		#       https://stackoverflow.com/a/70717331
		#       https://stackoverflow.com/a/60681762
		$new_row_obj | Export-Csv -LiteralPath $csvfilespec -Append -Force -UseQuotes AsNeeded
	} else {
		$new_row_obj | Export-Csv -LiteralPath $csvfilespec -Append -Force -NoTypeInformation
	}
}

function PurgeOldFiles() {
# PURPOSE:  Maintain the resized images folder; delete resized image files older than N hours 
# ARGS:     see param lines below
# RETURNS:  nothing
# ATTRIBUTIONS:  https://stackoverflow.com/a/46086926
	
    param(
		[string]$Pattern,  # folder location and filter for image files; a filespec like 'D:\myfolder\*.jpg'
		[int]$Hours        # maximum age for retained files; use 24 for 1 day, 168 for 1 week... 
	)
	
	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#Write-Host "Entered function = $fn_name()"

	# GET FILES MATCHING AGE REQUIREMENT
	$cur_list = get-childitem -Path $Pattern | Select Name
	$limit = (Get-Date).AddHours(-$Hours)
	$purge_obj = Get-ChildItem -Path $Pattern | Where-Object { !$_.PSIsContainer -and $_.CreationTime -lt $limit } | Select Name
	$purge_list = $purge_obj.Name
	
	if ($cur_list) {
		$cur_list_cnt = $cur_list.length
	} else {
		$cur_list_cnt = 0
	}

	if ($purge_list) {
		$purge_list_cnt = $purge_list.length
	} else {
		$purge_list_cnt = 0
	}

	# REMOVE FILES MATCHING AGE REQUIREMENT
	if ($purge_list_cnt -gt 0) {
		if ($Hours) {
			$null = Get-ChildItem -Path $Pattern | 
				Where-Object { !$_.PSIsContainer -and $_.CreationTime -lt $limit } | 
				Remove-Item -Force
		}
	}
	
	return  @($cur_list_cnt, $purge_list.Count)
}

# Start overall performance timer
$timer_total = [Diagnostics.Stopwatch]::StartNew()

if ($fromConsole) {
	Write-Host "=========================================================================================="
	Write-Host "Pushover Notification with Resized Image Attachment"
	Write-Host "Usage: $ScriptName -Msg [-Priority -AttachFile -Quality -Width -TargetKB -Demo]"
	Write-Host "Note:  using '-Quality 100 -Width 0' sends AttachFile without resizing"
	Write-Host "Note:  using '-Demo' skips notification & BI logfile posting (useful when tuning TargetKB)"
	Write-Host "------------------------------------------------------------------------------------------"
}

# HANDLE COMMANDLINE ARGS
$demo_flg = $false
if ($Msg) {
	$po_msg = $Msg
} else {
	$err_msg = "ERR/expecting 1 argument (Msg)"
	if ($post_to_BI_logfile_flg) {
		PostCustomBiLogEntry -MsgStr "$err_msg ($ScriptName)" -Level 2 -SilentFlg ($bg_msgs_flg -eq $false)
	}
	if ($fromConsole) {
		return "$err_msg`n"
	} else {
		msg.exe * /TIME:10 $err_msg
	}	
}
if ($Priority -or ($Priority -eq 0)) {
	if ($Priority -ne 0) {
		if (($Priority -ge -2) -and ($Priority -le 2)) {
			$po_priority = $Priority
		}
	}
}
$attach_path = $null
$variable_test = Get-Variable -Name AttachFile -Scope Local -ErrorAction SilentlyContinue
if ($variable_test -ne $null) {
	$attach_path = $AttachFile
}
#Write-Host "attach_path = '$attach_path'"
if ($Quality) {
	if ($Quality -ne '*') {
		if (([int]$Quality -ge 1) -and ([int]$Quality -le 100)) {
			$img_quality = [int]$Quality
		}
	}
}
if ($Width -or ([int]$Width -eq 0)) {
	if ($Width -ne '*') {
		$img_width = [int]$Width
	}
}
if ($TargetKB -or ([int]$TargetKB -eq 0)) {
	if ($TargetKB -ne '*') {
		$img_kilobytes = [int]$TargetKB
	}
}
$quiet_flg = $false
if ($Quiet) {
	if ($Quiet -match 'Y') {
		$quiet_flg = $true
	}
}
if ($quiet_flg) {
	$bg_msgs_flg = $false
}
$demo_flg = $false
if ($Demo) {
	if ($Demo -match 'Y') {
		$demo_flg = $true
	}
}

# ABORT IF PATHS NOT FOUND
if ($img_path) {
	if (-not(Test-Path -LiteralPath "$img_path") ) {
		$err_msg = "ERR/destination path does not exist!`n"
		if ($post_to_BI_logfile_flg) {
			PostCustomBiLogEntry -MsgStr "$err_msg ($ScriptName)" -Level 2 -SilentFlg ($bg_msgs_flg -eq $false)
		}
		if ($fromConsole) {
			return $err_msg
		} else {
			msg.exe * /TIME:10 "$err_msg`n`n(via script: '$ScriptName')"
		}	
	}
}
if ($attach_path) {
	if (-not(Test-Path -LiteralPath "$attach_path") ) {
		$err_msg = "ERR/alert image file not found! (expecting the full filespec)`n$attach_path"
		if ($post_to_BI_logfile_flg) {
			PostCustomBiLogEntry -MsgStr "$err_msg ($ScriptName)" -Level 2 -SilentFlg ($bg_msgs_flg -eq $false)
		}
		if ($fromConsole) {
			return $err_msg
		} else {
			msg.exe * /TIME:10 "$err_msg`n`n(via script: '$ScriptName')"
		}	
	}
}

# Create resized image's full filespec
if ($attach_path) {
	$full_filespec = (Resolve-Path $attach_path).Path
	$dot_idx = $full_filespec.LastIndexOf(".")
	if ($img_name_suffix) {
		$resized_image_path = $full_filespec.Substring(0,$dot_idx) + "_" + $img_name_suffix + $full_filespec.Substring($dot_idx,$full_filespec.Length - $dot_idx)
	} else {
		#$resized_image_path = $full_filespec.Substring(0,$dot_idx) + $full_filespec.Substring($dot_idx,$full_filespec.Length - $dot_idx)
		$resized_image_path = $full_filespec
	}

	# Change destination path, if requested
	$basename = Split-Path $attach_path -leaf
	$source_path = $full_filespec.Replace($basename, "")
	$resized_image_path = $resized_image_path.Replace($source_path, $img_path)

	# for debugging
	#Write-Host "source image full_filespec = '$full_filespec'"
	#Write-Host "source image filename = '$basename'"
	#Write-Host "source image source_path = '$source_path'"
	#Write-Host "resized_image_path = '$resized_image_path'"

	# Start resize performance timer
	$timer_resize = [Diagnostics.Stopwatch]::StartNew()

	# RESIZE ATTACHMENT
	$resize_ary = ResizeImage -SourceFilespec $attach_path -NewImagePath $resized_image_path -Quality $img_quality -Width $img_width -TargetKb $img_kilobytes -TargetKbMethod $targetkb_method -TargetKbWxHDelta $targetkb_wxh_step -TargetKbQualityDelta $targetkb_quality_step -SilentFlg ($bg_msgs_flg -eq $false)

	# Stop resize performance timer
	$timer_resize.stop()
	$resize_ms = [math]::Round($timer_resize.elapsed.totalmilliseconds,1).ToString("#.0")
}

# Start pushover performance timer
$timer_pusho = [Diagnostics.Stopwatch]::StartNew()

# SEND NOTIFICATION
if (!$demo_flg) {
	if ($Title -eq "*") {
		$po_title = "via $script:ScriptName"
	} else {
		$po_title = $Title
	}
	if ($attach_path) {
		if ($resize_ary[1] -eq 0) {
			# Try sending the original, unsized attachment
			SendPushover -MsgTitle $po_title -MsgStr $po_msg -AttachPath $attach_path -PriorityLevel $po_priority -SilentFlg ($bg_msgs_flg -eq $false)
			if ($fromConsole) {
				Write-Host "HEADS-UP: Requested attachment not resized; tried sending notification with unresized file"
			}
		} else {
			SendPushover -MsgTitle $po_title -MsgStr $po_msg -AttachPath $resized_image_path -PriorityLevel $po_priority -SilentFlg ($bg_msgs_flg -eq $false)
		}
	} else {
		SendPushover -MsgTitle $po_title -MsgStr $po_msg -AttachPath $null -PriorityLevel $po_priority -SilentFlg ($bg_msgs_flg -eq $false)
	}
}

# Stop pushover performance timer
$timer_pusho.stop()
$pusho_ms  = [math]::Round($timer_pusho.elapsed.totalmilliseconds,1).ToString("#.0")

# REMOVE OLD REDUCED IMAGE FILES
if (!$demo_flg) {
	$purge_filespec = $img_path + "*.jpg"
	#Write-Host "purge_filespec = '$purge_filespec'"
	$arr = PurgeOldFiles -Pattern $purge_filespec -Hours $img_lifetime_hours
	if ($arr[1] -gt 0) {			
		if ($fromConsole) {
			Write-Host ""
			Write-Host ("count of archived JPG files matching '$purge_filespec' = " + $arr[0])
			Write-Host ("count of archived JPG files purged (age >= $img_lifetime_hours hrs) = " + $arr[1])
		}
	}
}

# Stop overall performance timer
$timer_total.stop()
$total_ms  = [math]::Round($timer_total.elapsed.totalmilliseconds,1).ToString("#.0")

# Log script performance info
$csvfilespec = $img_path + $ScriptName + ".csv"
$timestamp   = Get-Date -Format "yyyy/MM/dd_HH:mm:ss"
$total_ms    = [decimal]$total_ms 
$pusho_ms    = [decimal]$pusho_ms 
if ($attach_path) {
	$size_before = [decimal]$resize_ary[0]
	$size_after  = [decimal]$resize_ary[1]
	$wxh_before  = [string]$resize_ary[2]
	$wxh_after   = [string]$resize_ary[3]
	$resize_ms   = [decimal]$resize_ms 
	LogPerformance $csvfilespec $attach_path $timestamp $size_before $size_after $wxh_before $wxh_after $total_ms $resize_ms $pusho_ms
} else {
	$size_before = 0
	$size_after  = 0
	$wxh_before  = ""
	$wxh_after   = ""
	$resize_ms   = 0 
	LogPerformance $csvfilespec 'NA' $timestamp $size_before $size_after $wxh_before $wxh_after $total_ms $resize_ms $pusho_ms
}

# DEV - performance notice
if ($fromConsole) {
	$perf_msg = "script execution time = " + [math]::Round($timer_total.elapsed.totalmilliseconds,1) + " msec"
	if ($demo_flg) {
		Write-Host "`n$perf_msg"
	} else {
		Write-Host $perf_msg
	}
}

$wxh_before     = $wxh_before.Replace(" x ", "x")
$wxh_after      = $wxh_after.Replace(" x ", "x")
$disp_total_ms  = [math]::Round($total_ms,0)
$disp_resize_ms = [math]::Round($resize_ms,0)
$disp_pusho_ms  = [math]::Round($pusho_ms,0)

# SHOW OPTIONAL MESSAGE WHEN RUNNING FROM BACKGROUND
if (!$fromConsole) {
	if ($bg_msgs_flg) {
		$msg_sec = 30
		$bg_msg =  $attach_path + 
			"`n`nWxH before/after  = $wxh_before / $wxh_after" +
			"`n`nSize before/after (KB) ... $size_before / $size_after" + 
			"`n`nexecution times (resize,pusho,total) ... $disp_resize_ms, $disp_pusho_ms, $disp_total_ms msec" + 
			"`n`n(this msg auto-closes in $msg_sec seconds)"
		msg.exe * /TIME:$msg_sec $bg_msg
	}
}

# POST MESSAGE TO BLUE IRIS LOGFILE
if (!$demo_flg) {
	if ($post_all_events) {
		$log_msg = "pushover resize [$wxh_before, $wxh_after; $size_before KB, $size_after KB; $disp_resize_ms msec, $disp_pusho_ms msec, $disp_total_ms msec] file=$attach_path"
		PostCustomBiLogEntry -MsgStr "$log_msg" -Level 0 -SilentFlg $true
	}
}

if ($fromConsole) {
	Write-Host ""
}

#Stop-Transcript